@@ -2,7 +2,6 @@
 ** LuaSQL, MySQL driver
 ** Authors:  Eduardo Quintao
 ** See Copyright Notice in license.html
-** $Id: ls_mysql.c,v 1.21 2006/01/25 20:49:30 tomas Exp $
 */
 
 #include <assert.h>
@@ -17,10 +16,10 @@
 #endif
 
 #include "mysql.h"
+#include "errmsg.h"
 
 #include "lua.h"
 #include "lauxlib.h"
-#include "compat-5.1.h"
 
 #include "luasql.h"
 
@@ -76,24 +75,9 @@ typedef struct {
 	int        numcols;            /* number of columns */
 	int        colnames, coltypes; /* reference to column information tables */
 	MYSQL_RES *my_res;
+	MYSQL 	  *my_conn;
 } cur_data;
 
-LUASQL_API int luaopen_luasqlmysql (lua_State *L);
-
-
-/*
-** Generates a driver error plus the error message from the database
-** The generated error message is preceded by LUASQL_PREFIX string
-*/
-static int luasql_failmessage(lua_State *L, const char *err, const char *m) {
-    lua_pushnil(L);
-	lua_pushstring(L, LUASQL_PREFIX);
-	lua_pushstring(L, err);
-    lua_pushstring(L, m);
-	lua_concat(L, 3);
-    return 2;
-}
-
 
 /*
 ** Check for valid environment.
@@ -195,6 +179,19 @@ static void create_colinfo (lua_State *L, cur_data *cur) {
 }
 
 
+/*
+** Closes the cursos and nullify all structure fields.
+*/
+static void cur_nullify (lua_State *L, cur_data *cur) {
+	/* Nullify structure fields. */
+	cur->closed = 1;
+	mysql_free_result(cur->my_res);
+	luaL_unref (L, LUA_REGISTRYINDEX, cur->conn);
+	luaL_unref (L, LUA_REGISTRYINDEX, cur->colnames);
+	luaL_unref (L, LUA_REGISTRYINDEX, cur->coltypes);
+}
+
+	
 /*
 ** Get another row of the given cursor.
 */
@@ -204,6 +201,7 @@ static int cur_fetch (lua_State *L) {
 	unsigned long *lengths;
 	MYSQL_ROW row = mysql_fetch_row(res);
 	if (row == NULL) {
+		cur_nullify (L, cur);
 		lua_pushnil(L);  /* no more results */
 		return 1;
 	}
@@ -248,6 +246,75 @@ static int cur_fetch (lua_State *L) {
 	}
 }
 
+/*
+** Get the next result from multiple statements
+*/
+static int cur_next_result (lua_State *L) {
+	cur_data *cur = getcursor (L);
+	MYSQL* con = cur->my_conn;
+	int status;
+	if(mysql_more_results(con)){
+		status = mysql_next_result(con);
+		if(status == 0){
+			mysql_free_result(cur->my_res);
+			cur->my_res = mysql_store_result(con);
+			if(cur->my_res != NULL){
+				lua_pushboolean(L, 1);
+				return 1;
+			}else{
+				lua_pushboolean(L, 0);
+				lua_pushinteger(L, mysql_errno(con));
+				lua_pushstring(L, mysql_error(con));
+				return 3;
+			}
+		}else{
+			lua_pushboolean(L, 0);
+			lua_pushinteger(L, status);
+			switch(status){
+				case CR_COMMANDS_OUT_OF_SYNC:
+					lua_pushliteral(L, "CR_COMMANDS_OUT_OF_SYNC");
+				break;
+				case CR_SERVER_GONE_ERROR:
+					lua_pushliteral(L, "CR_SERVER_GONE_ERROR");
+				break;
+				case CR_SERVER_LOST:
+					lua_pushliteral(L, "CR_SERVER_LOST");
+				break;
+				case CR_UNKNOWN_ERROR:
+					lua_pushliteral(L, "CR_UNKNOWN_ERROR");
+				break;
+				default:
+					lua_pushliteral(L, "Unknown");
+			}
+			return 3;
+		}
+	}else{
+		lua_pushboolean(L, 0);
+		lua_pushinteger(L, -1);
+		return 2;
+	}
+}
+
+/*
+** Check if next result is available
+*/
+static int cur_has_next_result (lua_State *L) {
+	cur_data *cur = getcursor (L);
+	lua_pushboolean(L, mysql_more_results(cur->my_conn));
+	return 1;
+}
+
+
+/*
+** Cursor object collector function
+*/
+static int cur_gc (lua_State *L) {
+	cur_data *cur = (cur_data *)luaL_checkudata (L, 1, LUASQL_CURSOR_MYSQL);
+	if (cur != NULL && !(cur->closed))
+		cur_nullify (L, cur);
+	return 0;
+}
+
 
 /*
 ** Close the cursor on top of the stack.
@@ -260,14 +327,7 @@ static int cur_close (lua_State *L) {
 		lua_pushboolean (L, 0);
 		return 1;
 	}
-
-	/* Nullify structure fields. */
-	cur->closed = 1;
-	mysql_free_result(cur->my_res);
-	luaL_unref (L, LUA_REGISTRYINDEX, cur->conn);
-	luaL_unref (L, LUA_REGISTRYINDEX, cur->colnames);
-	luaL_unref (L, LUA_REGISTRYINDEX, cur->coltypes);
-
+	cur_nullify (L, cur);
 	lua_pushboolean (L, 1);
 	return 1;
 }
@@ -313,15 +373,26 @@ static int cur_getcoltypes (lua_State *L) {
 ** Push the number of rows.
 */
 static int cur_numrows (lua_State *L) {
-	lua_pushnumber (L, (lua_Number)mysql_num_rows (getcursor(L)->my_res));
+	lua_pushinteger (L, (lua_Number)mysql_num_rows (getcursor(L)->my_res));
 	return 1;
 }
 
 
+/*
+** Seeks to an arbitrary row in a query result set.
+*/
+static int cur_seek (lua_State *L) {
+	cur_data *cur = getcursor (L);
+	lua_Integer rownum = luaL_checkinteger (L, 2);
+	mysql_data_seek (cur->my_res, rownum);
+	return 0;
+}
+
+
 /*
 ** Create a new Cursor object and push it on top of the stack.
 */
-static int create_cursor (lua_State *L, int conn, MYSQL_RES *result, int cols) {
+static int create_cursor (lua_State *L, MYSQL *my_conn, int conn, MYSQL_RES *result, int cols) {
 	cur_data *cur = (cur_data *)lua_newuserdata(L, sizeof(cur_data));
 	luasql_setmeta (L, LUASQL_CURSOR_MYSQL);
 
@@ -332,6 +403,7 @@ static int create_cursor (lua_State *L, int conn, MYSQL_RES *result, int cols) {
 	cur->colnames = LUA_NOREF;
 	cur->coltypes = LUA_NOREF;
 	cur->my_res = result;
+	cur->my_conn = my_conn;
 	lua_pushvalue (L, conn);
 	cur->conn = luaL_ref (L, LUA_REGISTRYINDEX);
 
@@ -339,6 +411,18 @@ static int create_cursor (lua_State *L, int conn, MYSQL_RES *result, int cols) {
 }
 
 
+static int conn_gc (lua_State *L) {
+	conn_data *conn=(conn_data *)luaL_checkudata(L, 1, LUASQL_CONNECTION_MYSQL);
+	if (conn != NULL && !(conn->closed)) {
+		/* Nullify structure fields. */
+		conn->closed = 1;
+		luaL_unref (L, LUA_REGISTRYINDEX, conn->env);
+		mysql_close (conn->my_conn);
+	}
+	return 0;
+}
+
+
 /*
 ** Close a Connection object.
 */
@@ -349,15 +433,48 @@ static int conn_close (lua_State *L) {
 		lua_pushboolean (L, 0);
 		return 1;
 	}
-
-	/* Nullify structure fields. */
-	conn->closed = 1;
-	luaL_unref (L, LUA_REGISTRYINDEX, conn->env);
-	mysql_close (conn->my_conn);
+	conn_gc (L);
 	lua_pushboolean (L, 1);
 	return 1;
 }
 
+/*
+** Ping connection.
+*/
+static int conn_ping (lua_State *L) {
+	conn_data *conn=(conn_data *)luaL_checkudata(L, 1, LUASQL_CONNECTION_MYSQL);
+	luaL_argcheck (L, conn != NULL, 1, LUASQL_PREFIX"connection expected");
+	if (conn->closed) {
+		lua_pushboolean (L, 0);
+		return 1;
+	}
+	if (mysql_ping (conn->my_conn) == 0) {
+		lua_pushboolean (L, 1);
+		return 1;
+	} else if (mysql_errno (conn->my_conn) == CR_SERVER_GONE_ERROR) {
+		lua_pushboolean (L, 0);
+		return 1;
+	}
+	luaL_error(L, mysql_error(conn->my_conn));
+	return 0;
+}
+
+
+static int escape_string (lua_State *L) {
+  size_t size, new_size;
+  conn_data *conn = getconnection (L);
+  const char *from = luaL_checklstring(L, 2, &size);
+  char *to;
+  to = (char*)malloc(sizeof(char) * (2 * size + 1));
+  if(to) {
+    new_size = mysql_real_escape_string(conn->my_conn, to, from, size);
+    lua_pushlstring(L, to, new_size);
+    free(to);
+    return 1;
+  }
+  luaL_error(L, "could not allocate escaped string");
+  return 0;
+}
 
 /*
 ** Execute an SQL statement.
@@ -366,27 +483,27 @@ static int conn_close (lua_State *L) {
 */
 static int conn_execute (lua_State *L) {
 	conn_data *conn = getconnection (L);
-	const char *statement = luaL_checkstring (L, 2);
-	unsigned long st_len = strlen(statement);
+	size_t st_len;
+	const char *statement = luaL_checklstring (L, 2, &st_len);
 	if (mysql_real_query(conn->my_conn, statement, st_len)) 
 		/* error executing query */
-		return luasql_failmessage(L, "Error executing query. MySQL: ", mysql_error(conn->my_conn));
+		return luasql_failmsg(L, "error executing query. MySQL: ", mysql_error(conn->my_conn));
 	else
 	{
 		MYSQL_RES *res = mysql_store_result(conn->my_conn);
 		unsigned int num_cols = mysql_field_count(conn->my_conn);
 
 		if (res) { /* tuples returned */
-			return create_cursor (L, 1, res, num_cols);
+			return create_cursor (L, conn->my_conn, 1, res, num_cols);
 		}
 		else { /* mysql_use_result() returned nothing; should it have? */
 			if(num_cols == 0) { /* no tuples returned */
             	/* query does not return data (it was not a SELECT) */
-				lua_pushnumber(L, mysql_affected_rows(conn->my_conn));
+				lua_pushinteger(L, mysql_affected_rows(conn->my_conn));
 				return 1;
         	}
 			else /* mysql_use_result() should have returned data */
-				return luasql_failmessage(L, "Error retrieving result. MySQL: ", mysql_error(conn->my_conn));
+				return luasql_failmsg(L, "error retrieving result. MySQL: ", mysql_error(conn->my_conn));
 		}
 	}
 }
@@ -397,18 +514,19 @@ static int conn_execute (lua_State *L) {
 */
 static int conn_commit (lua_State *L) {
 	conn_data *conn = getconnection (L);
-	mysql_commit(conn->my_conn);
-	return 0;
+	lua_pushboolean(L, !mysql_commit(conn->my_conn));
+	return 1;
 }
 
 
+
 /*
 ** Rollback the current transaction.
 */
 static int conn_rollback (lua_State *L) {
 	conn_data *conn = getconnection (L);
-	mysql_rollback(conn->my_conn);
-	return 0;
+	lua_pushboolean(L, !mysql_rollback(conn->my_conn));
+	return 1;
 }
 
 
@@ -423,10 +541,20 @@ static int conn_setautocommit (lua_State *L) {
 	else {
 		mysql_autocommit(conn->my_conn, 0);
 	}
-	return 0;
+	lua_pushboolean(L, 1);
+	return 1;
 }
 
 
+/*
+** Get Last auto-increment id generated
+*/
+static int conn_getlastautoid (lua_State *L) {
+  conn_data *conn = getconnection(L);
+  lua_pushinteger(L, mysql_insert_id(conn->my_conn));
+  return 1;
+}
+
 /*
 ** Create a new Connection object and push it on top of the stack.
 */
@@ -454,27 +582,39 @@ static int env_connect (lua_State *L) {
 	const char *username = luaL_optstring(L, 3, NULL);
 	const char *password = luaL_optstring(L, 4, NULL);
 	const char *host = luaL_optstring(L, 5, NULL);
-	const int port = luaL_optint(L, 6, 0);
+	const int port = luaL_optinteger(L, 6, 0);
+	const char *unix_socket = luaL_optstring(L, 7, NULL);
+	const long client_flag = (long)luaL_optinteger(L, 8, 0);
 	MYSQL *conn;
 	getenvironment(L); /* validade environment */
 
 	/* Try to init the connection object. */
 	conn = mysql_init(NULL);
 	if (conn == NULL)
-		return luasql_faildirect(L, LUASQL_PREFIX"Error connecting: Out of memory.");
+		return luasql_faildirect(L, "error connecting: Out of memory.");
 
 	if (!mysql_real_connect(conn, host, username, password, 
-		sourcename, port, NULL, 0))
+		sourcename, port, unix_socket, client_flag))
 	{
 		char error_msg[100];
 		strncpy (error_msg,  mysql_error(conn), 99);
 		mysql_close (conn); /* Close conn if connect failed */
-		return luasql_failmessage (L, "Error connecting to database. MySQL: ", error_msg);
+		return luasql_failmsg (L, "error connecting to database. MySQL: ", error_msg);
 	}
 	return create_connection(L, 1, conn);
 }
 
 
+/*
+**
+*/
+static int env_gc (lua_State *L) {
+	env_data *env= (env_data *)luaL_checkudata (L, 1, LUASQL_ENVIRONMENT_MYSQL);	if (env != NULL && !(env->closed))
+		env->closed = 1;
+	return 0;
+}
+
+
 /*
 ** Close environment object.
 */
@@ -485,7 +625,7 @@ static int env_close (lua_State *L) {
 		lua_pushboolean (L, 0);
 		return 1;
 	}
-
+	mysql_library_end();
 	env->closed = 1;
 	lua_pushboolean (L, 1);
 	return 1;
@@ -496,25 +636,34 @@ static int env_close (lua_State *L) {
 ** Create metatables for each class of object.
 */
 static void create_metatables (lua_State *L) {
-    struct luaL_reg environment_methods[] = {
+    struct luaL_Reg environment_methods[] = {
+        {"__gc", env_gc},
         {"close", env_close},
         {"connect", env_connect},
 		{NULL, NULL},
 	};
-    struct luaL_reg connection_methods[] = {
+    struct luaL_Reg connection_methods[] = {
+        {"__gc", conn_gc},
         {"close", conn_close},
+        {"ping", conn_ping},
+        {"escape", escape_string},
         {"execute", conn_execute},
         {"commit", conn_commit},
         {"rollback", conn_rollback},
         {"setautocommit", conn_setautocommit},
+		{"getlastautoid", conn_getlastautoid},
 		{NULL, NULL},
     };
-    struct luaL_reg cursor_methods[] = {
+    struct luaL_Reg cursor_methods[] = {
+        {"__gc", cur_gc},
         {"close", cur_close},
         {"getcolnames", cur_getcolnames},
         {"getcoltypes", cur_getcoltypes},
         {"fetch", cur_fetch},
         {"numrows", cur_numrows},
+        {"seek", cur_seek},
+		{"nextresult", cur_next_result},
+		{"hasnextresult", cur_has_next_result},
 		{NULL, NULL},
     };
 	luasql_createmeta (L, LUASQL_ENVIRONMENT_MYSQL, environment_methods);
@@ -541,16 +690,21 @@ static int create_environment (lua_State *L) {
 ** Creates the metatables for the objects and registers the
 ** driver open method.
 */
-LUASQL_API int luaopen_luasqlmysql (lua_State *L) { 
-	struct luaL_reg driver[] = {
+LUASQL_API int luaopen_luasql_mysql (lua_State *L) { 
+	struct luaL_Reg driver[] = {
 		{"mysql", create_environment},
 		{NULL, NULL},
 	};
 	create_metatables (L);
-	luaL_openlib (L, LUASQL_TABLENAME, driver, 0);
+	lua_newtable(L);
+	luaL_setfuncs(L, driver, 0);
 	luasql_set_info (L);
-    lua_pushliteral (L, "_MYSQLVERSION");
-    lua_pushliteral (L, MYSQL_SERVER_VERSION);
+    lua_pushliteral (L, "_CLIENTVERSION");
+#ifdef MARIADB_CLIENT_VERSION_STR
+lua_pushliteral (L, MARIADB_CLIENT_VERSION_STR);
+#else
+lua_pushliteral (L, MYSQL_SERVER_VERSION);
+#endif
     lua_settable (L, -3);
 	return 1;
 }
