@@ -2,7 +2,6 @@
 ** LuaSQL, Oracle driver
 ** Authors: Tomas Guisasola, Leonardo Godinho
 ** See Copyright Notice in license.html
-** $Id: ls_oci8.c,v 1.25 2005/11/01 19:28:17 tuler Exp $
 */
 
 #include <assert.h>
@@ -18,7 +17,6 @@
 
 #include "lua.h"
 #include "lauxlib.h"
-#include "compat-5.1.h"
 
 #include "luasql.h"
 
@@ -124,7 +122,7 @@ int checkerr (lua_State *L, sword status, OCIError *errhp) {
 			lua_pushnil (L); /* !!!!!!!!!!!!! */
 			break;
 		case OCI_SUCCESS_WITH_INFO:
-			lua_pushstring (L, LUASQL_PREFIX"Sucesso com info!");
+			lua_pushstring (L, LUASQL_PREFIX"Success with info!");
 			break;
 		case OCI_NEED_DATA:
 			lua_pushstring (L, LUASQL_PREFIX"OCI_NEED_DATA");
@@ -189,8 +187,6 @@ static int alloc_column_buffer (lua_State *L, cur_data *cur, int i) {
 		cur->errhp), cur->errhp);
 
 	switch (col->type) {
-		case SQLT_CHR:
-		case SQLT_STR:
 		case SQLT_VCS:
 		case SQLT_AFC:
 		case SQLT_AVC:
@@ -203,6 +199,17 @@ static int alloc_column_buffer (lua_State *L, cur_data *cur, int i) {
 				SQLT_STR /*col->type*/, (dvoid *)&(col->null), (ub2 *)0,
 				(ub2 *)0, (ub4) OCI_DEFAULT), cur->errhp);
 			break;
+		case SQLT_CHR:
+		case SQLT_STR:
+			ASSERT (L, OCIAttrGet (param, OCI_DTYPE_PARAM,
+				(dvoid *)&(col->max), 0, OCI_ATTR_DATA_SIZE,
+				cur->errhp), cur->errhp);
+				col->val.s = calloc (col->max * 2 + 1, sizeof(col->val.s));
+			ASSERT (L, OCIDefineByPos (cur->stmthp, &(col->define),
+				cur->errhp, (ub4)i, col->val.s, col->max * 2 + 1,
+				SQLT_STR /*col->type*/, (dvoid *)&(col->null), (ub2 *)0,
+				(ub2 *)0, (ub4) OCI_DEFAULT), cur->errhp);
+			break;
 		case SQLT_NUM:
 		case SQLT_FLT:
 		case SQLT_INT:
@@ -662,7 +669,7 @@ static int conn_commit (lua_State *L) {
 	ASSERT (L, OCITransCommit (conn->svchp, conn->errhp, OCI_DEFAULT),
 		conn->errhp);
 /*
-	if (conn->auto_commit == 0) 
+	if (conn->auto_commit == 0)
 		ASSERT (L, OCITransStart (conn->svchp, conn->errhp...
 */
 	return 0;
@@ -677,8 +684,8 @@ static int conn_rollback (lua_State *L) {
 	ASSERT (L, OCITransRollback (conn->svchp, conn->errhp, OCI_DEFAULT),
 		conn->errhp);
 /*
-	if (conn->auto_commit == 0) 
-		sql_begin(conn); 
+	if (conn->auto_commit == 0)
+		sql_begin(conn);
 */
 	return 0;
 }
@@ -701,7 +708,8 @@ static int conn_setautocommit (lua_State *L) {
 		conn->auto_commit = 0;
 		/* sql_begin(conn);*/
 	}
-	return 0;
+	lua_pushboolean(L, 1);
+	return 1;
 }
 
 
@@ -796,7 +804,7 @@ static int create_environment (lua_State *L) {
 			(void (*)(dvoid *, dvoid *)) 0,
 			(size_t) 0,
 			(dvoid **) 0))
-		luasql_faildirect (L, LUASQL_PREFIX"couldn't create environment");
+		luasql_faildirect (L, "couldn't create environment");
 	/* error handler */
 	ASSERT (L, OCIHandleAlloc((dvoid *) env->envhp,
 		(dvoid **) &(env->errhp), /* !!! */
@@ -809,12 +817,14 @@ static int create_environment (lua_State *L) {
 ** Create metatables for each class of object.
 */
 static void create_metatables (lua_State *L) {
-	struct luaL_reg environment_methods[] = {
+	struct luaL_Reg environment_methods[] = {
+		{"__gc", env_close}, /* Should this method be changed? */
 		{"close", env_close},
 		{"connect", env_connect},
 		{NULL, NULL},
 	};
-	struct luaL_reg connection_methods[] = {
+	struct luaL_Reg connection_methods[] = {
+		{"__gc", conn_close}, /* Should this method be changed? */
 		{"close", conn_close},
 		{"execute", conn_execute},
 		{"commit", conn_commit},
@@ -822,7 +832,8 @@ static void create_metatables (lua_State *L) {
 		{"setautocommit", conn_setautocommit},
 		{NULL, NULL},
 	};
-	struct luaL_reg cursor_methods[] = {
+	struct luaL_Reg cursor_methods[] = {
+		{"__gc", cur_close}, /* Should this method be changed? */
 		{"close", cur_close},
 		{"getcolnames", cur_getcolnames},
 		{"getcoltypes", cur_getcoltypes},
@@ -841,13 +852,14 @@ static void create_metatables (lua_State *L) {
 ** Creates the metatables for the objects and registers the
 ** driver open method.
 */
-LUASQL_API int luaopen_luasqloci8 (lua_State *L) {
-	struct luaL_reg driver[] = {
+LUASQL_API int luaopen_luasql_oci8 (lua_State *L) {
+	struct luaL_Reg driver[] = {
 		{"oci8", create_environment},
 		{NULL, NULL},
 	};
 	create_metatables (L);
-	luaL_openlib (L, LUASQL_TABLENAME, driver, 0);
+	lua_newtable (L);
+	luaL_setfuncs (L, driver, 0);
 	luasql_set_info (L);
 	return 1;
 }
