@@ -3,7 +3,6 @@
 ** Authors: Pedro Rabinovitch, Roberto Ierusalimschy, Carlos Cassino
 ** Tomas Guisasola, Eduardo Quintao
 ** See Copyright Notice in license.html
-** $Id: ls_postgres.c,v 1.9 2007/06/18 01:22:45 tomas Exp $
 */
 
 #include <assert.h>
@@ -16,9 +15,6 @@
 
 #include "lua.h"
 #include "lauxlib.h"
-#if ! defined (LUA_VERSION_NUM) || LUA_VERSION_NUM < 501
-#include "compat-5.1.h"
-#endif
 
 
 #include "luasql.h"
@@ -53,9 +49,6 @@ typedef struct {
 typedef void (*creator) (lua_State *L, cur_data *cur);
 
 
-LUASQL_API int luaopen_luasql_postgres(lua_State *L);
-
-
 /*
 ** Check for valid environment.
 */
@@ -100,6 +93,19 @@ static void pushvalue (lua_State *L, PGresult *res, int tuple, int i) {
 }
 
 
+/*
+** Closes the cursor and nullify all structure fields.
+*/
+static void cur_nullify (lua_State *L, cur_data *cur) {
+	/* Nullify structure fields. */
+	cur->closed = 1;
+	PQclear(cur->pg_res);
+	luaL_unref (L, LUA_REGISTRYINDEX, cur->conn);
+	luaL_unref (L, LUA_REGISTRYINDEX, cur->colnames);
+	luaL_unref (L, LUA_REGISTRYINDEX, cur->coltypes);
+}
+
+
 /*
 ** Get another row of the given cursor.
 */
@@ -109,6 +115,7 @@ static int cur_fetch (lua_State *L) {
 	int tuple = cur->curr_tuple;
 
 	if (tuple >= PQntuples(cur->pg_res)) {
+		cur_nullify (L, cur);
 		lua_pushnil(L);  /* no more results */
 		return 1;
 	}
@@ -144,8 +151,21 @@ static int cur_fetch (lua_State *L) {
 
 
 /*
-** Close the cursor on top of the stack.
-** Return 1
+** Cursor object collector function
+*/
+static int cur_gc (lua_State *L) {
+	cur_data *cur = (cur_data *)luaL_checkudata (L, 1, LUASQL_CURSOR_PG);
+	if (cur != NULL && !(cur->closed))
+		cur_nullify (L, cur);
+	return 0;
+}
+
+
+/*
+** Closes the cursor on top of the stack.
+** Returns true in case of success, or false in case the cursor was
+** already closed.
+** Throws an error if the argument is not a cursor.
 */
 static int cur_close (lua_State *L) {
 	cur_data *cur = (cur_data *)luaL_checkudata (L, 1, LUASQL_CURSOR_PG);
@@ -154,14 +174,7 @@ static int cur_close (lua_State *L) {
 		lua_pushboolean (L, 0);
 		return 1;
 	}
-
-	/* Nullify structure fields. */
-	cur->closed = 1;
-	PQclear(cur->pg_res);
-	luaL_unref (L, LUA_REGISTRYINDEX, cur->conn);
-	luaL_unref (L, LUA_REGISTRYINDEX, cur->colnames);
-	luaL_unref (L, LUA_REGISTRYINDEX, cur->coltypes);
-
+	cur_nullify (L, cur); /* == cur_gc (L); */
 	lua_pushboolean (L, 1);
 	return 1;
 }
@@ -173,10 +186,10 @@ static int cur_close (lua_State *L) {
 static char *getcolumntype (PGconn *conn, PGresult *result, int i, char *buff) {
 	Oid codigo = PQftype (result, i);
 	char stmt[100];
-  	PGresult *res;
+	PGresult *res;
 
- 	sprintf (stmt, "select typname from pg_type where oid = %d", codigo);
-  	res = PQexec(conn, stmt);
+	sprintf (stmt, "select typname from pg_type where oid = %d", codigo);
+	res = PQexec(conn, stmt);
 	strcpy (buff, "undefined");
 
 	if (PQresultStatus (res) == PGRES_TUPLES_OK) {
@@ -303,7 +316,7 @@ static void sql_commit(conn_data *conn) {
 
 
 static void sql_begin(conn_data *conn) {
-	PQclear(PQexec(conn->pg_conn, "BEGIN")); 
+	PQclear(PQexec(conn->pg_conn, "BEGIN"));
 }
 
 
@@ -313,7 +326,25 @@ static void sql_rollback(conn_data *conn) {
 
 
 /*
-** Close a Connection object.
+** Connection object collector function
+*/
+static int conn_gc (lua_State *L) {
+	conn_data *conn = (conn_data *)luaL_checkudata (L, 1, LUASQL_CONNECTION_PG);
+	if (conn != NULL && !(conn->closed)) {
+		/* Nullify structure fields. */
+		conn->closed = 1;
+		luaL_unref (L, LUA_REGISTRYINDEX, conn->env);
+		PQfinish (conn->pg_conn);
+	}
+	return 0;
+}
+
+
+/*
+** Closes the connection on top of the stack.
+** Returns true in case of success, or false in case the connection was
+** already closed.
+** Throws an error if the argument is not a connection.
 */
 static int conn_close (lua_State *L) {
 	conn_data *conn = (conn_data *)luaL_checkudata (L, 1, LUASQL_CONNECTION_PG);
@@ -322,11 +353,7 @@ static int conn_close (lua_State *L) {
 		lua_pushboolean (L, 0);
 		return 1;
 	}
-
-	/* Nullify structure fields. */
-	conn->closed = 1;
-	luaL_unref (L, LUA_REGISTRYINDEX, conn->env);
-	PQfinish (conn->pg_conn);
+	conn_gc (L);
 	lua_pushboolean (L, 1);
 	return 1;
 }
@@ -340,14 +367,38 @@ static int conn_escape (lua_State *L) {
 	conn_data *conn = getconnection (L);
 	size_t len;
 	const char *from = luaL_checklstring (L, 2, &len);
-	char to[len*2+1];
 	int error;
+	int ret = 1;
+	luaL_Buffer b;
+	char *to;
+#if !defined(LUA_VERSION_NUM) || (LUA_VERSION_NUM == 501)
+	/* Lua 5.0 and 5.1 */
+	luaL_buffinit (L, &b);
+	do {
+		int max = LUAL_BUFFERSIZE / 2;
+		size_t bytes_copied;
+		size_t this_len = (len > max) ? max : len;
+		to = luaL_prepbuffer (&b);
+		bytes_copied = PQescapeStringConn (conn->pg_conn, to, from, this_len, &error);
+		if (error != 0) { /* failed ! */
+			return luasql_failmsg (L, "cannot escape string. PostgreSQL: ", PQerrorMessage (conn->pg_conn));
+		}
+		luaL_addsize (&b, bytes_copied);
+		from += this_len;
+		len -= this_len;
+	} while (len > 0);
+	luaL_pushresult (&b);
+#else
+	/* Lua 5.2 and 5.3 */
+	to = luaL_buffinitsize (L, &b, 2*len+1);
 	len = PQescapeStringConn (conn->pg_conn, to, from, len, &error);
 	if (error == 0) { /* success ! */
-		lua_pushlstring (L, to, len);
-		return 1;
-	} else
-		return luasql_faildirect (L, PQerrorMessage (conn->pg_conn));
+		luaL_pushresultsize (&b, len);
+	} else {
+		ret = luasql_failmsg (L, "cannot escape string. PostgreSQL: ", PQerrorMessage (conn->pg_conn));
+	}
+#endif
+	return ret;
 }
 
 
@@ -372,7 +423,7 @@ static int conn_execute (lua_State *L) {
 	else {
 		/* error */
 		PQclear (res);
-		return luasql_faildirect(L, PQerrorMessage(conn->pg_conn));
+		return luasql_failmsg(L, "error executing statement. PostgreSQL: ", PQerrorMessage(conn->pg_conn));
 	}
 }
 
@@ -384,7 +435,7 @@ static int conn_commit (lua_State *L) {
 	conn_data *conn = getconnection (L);
 	sql_commit(conn);
 	if (conn->auto_commit == 0) {
-		sql_begin(conn); 
+		sql_begin(conn);
 		lua_pushboolean (L, 1);
 	} else
 		lua_pushboolean (L, 0);
@@ -399,7 +450,7 @@ static int conn_rollback (lua_State *L) {
 	conn_data *conn = getconnection (L);
 	sql_rollback(conn);
 	if (conn->auto_commit == 0) {
-		sql_begin(conn); 
+		sql_begin(conn);
 		lua_pushboolean (L, 1);
 	} else
 		lua_pushboolean (L, 0);
@@ -446,44 +497,50 @@ static int create_connection (lua_State *L, int env, PGconn *const pg_conn) {
 
 
 static void notice_processor (void *arg, const char *message) {
-  (void)arg; (void)message;
-  /* arg == NULL */
+	(void)arg; (void)message;
+	/* arg == NULL */
 }
 
 
 /*
 ** Connects to a data source.
-** This driver provides two ways to connect to a data source:
-** (1) giving the connection parameters as a set of pairs separated
-**     by whitespaces in a string (first method parameter)
-** (2) giving one string for each connection parameter, said
-**     datasource, username, password, host and port.
 */
 static int env_connect (lua_State *L) {
 	const char *sourcename = luaL_checkstring(L, 2);
+	const char *username = luaL_optstring(L, 3, NULL);
+	const char *password = luaL_optstring(L, 4, NULL);
+	const char *pghost = luaL_optstring(L, 5, NULL);
+	const char *pgport = luaL_optstring(L, 6, NULL);
 	PGconn *conn;
 	getenvironment (L);	/* validate environment */
-	if ((lua_gettop (L) == 2) && (strchr (sourcename, '=') != NULL))
-		conn = PQconnectdb (sourcename);
-	else {
-		const char *username = luaL_optstring(L, 3, NULL);
-		const char *password = luaL_optstring(L, 4, NULL);
-		const char *pghost = luaL_optstring(L, 5, NULL);
-		const char *pgport = luaL_optstring(L, 6, NULL);
+	conn = PQsetdbLogin(pghost, pgport, NULL, NULL, sourcename, username, password);
 
-		conn = PQsetdbLogin(pghost, pgport, NULL, NULL,
-			sourcename, username, password);
+	if (PQstatus(conn) == CONNECTION_BAD) {
+		int rc = luasql_failmsg(L, "error connecting to database. PostgreSQL: ", PQerrorMessage(conn));
+		PQfinish(conn);
+		return rc;
 	}
-
-	if (PQstatus(conn) == CONNECTION_BAD)
-		return luasql_faildirect(L, LUASQL_PREFIX"Error connecting to database.");
 	PQsetNoticeProcessor(conn, notice_processor, NULL);
 	return create_connection(L, 1, conn);
 }
 
 
 /*
-** Close environment object.
+** Environment object collector function.
+*/
+static int env_gc (lua_State *L) {
+	env_data *env = (env_data *)luaL_checkudata (L, 1, LUASQL_ENVIRONMENT_PG);
+	if (env != NULL && !(env->closed))
+		env->closed = 1;
+	return 0;
+}
+
+
+/*
+** Closes the environment on top of the stack.
+** Returns true in case of success, or false in case the environment was
+** already closed.
+** Throws an error if the argument is not an environment.
 */
 static int env_close (lua_State *L) {
 	env_data *env = (env_data *)luaL_checkudata (L, 1, LUASQL_ENVIRONMENT_PG);
@@ -492,8 +549,7 @@ static int env_close (lua_State *L) {
 		lua_pushboolean (L, 0);
 		return 1;
 	}
-
-	env->closed = 1;
+	env_gc (L);
 	lua_pushboolean (L, 1);
 	return 1;
 }
@@ -504,28 +560,31 @@ static int env_close (lua_State *L) {
 ** Create metatables for each class of object.
 */
 static void create_metatables (lua_State *L) {
-    struct luaL_reg environment_methods[] = {
-        {"close", env_close},
-        {"connect", env_connect},
+	struct luaL_Reg environment_methods[] = {
+		{"__gc",    env_gc},
+		{"close",   env_close},
+		{"connect", env_connect},
 		{NULL, NULL},
 	};
-    struct luaL_reg connection_methods[] = {
-        {"close", conn_close},
-        {"escape", conn_escape},
-        {"execute", conn_execute},
-        {"commit", conn_commit},
-        {"rollback", conn_rollback},
-        {"setautocommit", conn_setautocommit},
+	struct luaL_Reg connection_methods[] = {
+		{"__gc",          conn_gc},
+		{"close",         conn_close},
+		{"escape",        conn_escape},
+		{"execute",       conn_execute},
+		{"commit",        conn_commit},
+		{"rollback",      conn_rollback},
+		{"setautocommit", conn_setautocommit},
 		{NULL, NULL},
-    };
-    struct luaL_reg cursor_methods[] = {
-        {"close", cur_close},
-        {"getcolnames", cur_getcolnames},
-        {"getcoltypes", cur_getcoltypes},
-        {"fetch", cur_fetch},
-		{"numrows", cur_numrows},
+	};
+	struct luaL_Reg cursor_methods[] = {
+		{"__gc",        cur_gc},
+		{"close",       cur_close},
+		{"getcolnames", cur_getcolnames},
+		{"getcoltypes", cur_getcoltypes},
+		{"fetch",       cur_fetch},
+		{"numrows",     cur_numrows},
 		{NULL, NULL},
-    };
+	};
 	luasql_createmeta (L, LUASQL_ENVIRONMENT_PG, environment_methods);
 	luasql_createmeta (L, LUASQL_CONNECTION_PG, connection_methods);
 	luasql_createmeta (L, LUASQL_CURSOR_PG, cursor_methods);
@@ -550,12 +609,18 @@ static int create_environment (lua_State *L) {
 ** driver open method.
 */
 LUASQL_API int luaopen_luasql_postgres (lua_State *L) {
-    struct luaL_reg driver[] = {
+	struct luaL_Reg driver[] = {
 		{"postgres", create_environment},
 		{NULL, NULL},
 	};
 	create_metatables (L);
-	luaL_openlib (L, LUASQL_TABLENAME, driver, 0);
+	lua_newtable (L);
+	luaL_setfuncs (L, driver, 0);
 	luasql_set_info (L);
+#if defined(PQlibVersion)
+	lua_pushliteral (L, "_CLIENTVERSION");
+	lua_pushinteger (L, PQlibVersion());
+	lua_settable (L, -3);
+#endif
 	return 1;
 }
